#/bin/bash

# This shell script prints a logarithmic spiral to the terminal screen
# Copyright (C) 2013 Istvan Szantai <szantaii@sidenote.hu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program (license.txt).
# If not, see <http://www.gnu.org/licenses/>.


# Function that echoes the absolute value of an integer
# abs -1 will echo 1
abs()
{
	if (($1 < 0))
	then
		echo $(($1 * -1))
	else
		echo $1
	fi
}

# Declaration and initialization of necessary variables
x=0
y=0
abs_x=0
abs_y=0
a=0.1
b=0.1
pi=3.14159265358979323844
term_width=$(tput cols)
term_height=$(tput lines)

# Declare an associative array that simulates a two dimensional array
# which is used to print only one character per screen coordinate (x,y)
declare -A grid

for ((i=0; i < term_width; i++))
do
	for ((j=0; j < term_height; j++))
	do
		grid["${i},${j}"]=0
	done
done

# Clear the terminal screen
tput reset

# Loop until the spiral reaches either side of the terminal screen
for ((i=0; abs_x < $((term_width / 2)) && abs_y < $((term_height / 2)); i++))
do
	# Convert the current angle (in degrees) to radians
	t=$(echo "scale=20; ${i} * ${pi} / 180" | bc -l)
	
	# Calculate integer 'x' and 'y' coordinates of the logarithmic
	# spiral based on the following two equations:
	#     x(t) = a * e^(b * t) * cos(t)
	#     y(t) = a * e^(b * t) * sin(t)
	x=$(echo "scale=20; ${a} * e(${b} * ${t}) * c(-${t})" \
		| bc -l | xargs printf "%1.0f\n")
	y=$(echo "scale=20; ${a} * e(${b} * ${t}) * s(-${t})" \
		| bc -l | xargs printf "%1.0f\n")
	
	# Get the absolute value
	abs_x=$(abs $x)
	abs_y=$(abs $y)
	
	# Check if the current grid index exists
	current_index="$((${term_width} / 2 + ${x})),$((${term_height} / 2 + ${y}))"
	exists=${grid["${current_index}"]}
	if [[ $exists == "" ]]
	then
		exists=0
	else
		exists=1
	fi
	
	# If the current grid index is valid
	if (( $exists == 1 ))
	then
		# If the current grid element is not printed yet then
		# place the cursor, print a character to the screen,
		# and mark the current grid element as printed
		if (( ${grid["${current_index}"]} == 0 ))
		then
			tput cup $((term_height / 2 + y)) $((term_width / 2 + x))
			printf "o"
			grid["${current_index}"]=1
		fi
	fi
done

# Place the cursor to the lower right corner
# of the terminal window and wait for keypress
tput cup $term_height $term_width
read -sn1
printf "\n"

